/* GENERAL
* Version 2.50.2-7793
* Created 2017-02-15 02:00
*/
"use strict";

/**
 * SVG component represents BarGraph.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.BarGraph} New SVG BarGraph component
 */
REX.UI.SVG.BarGraph = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var r_min = parseFloat(that.check($o.rangeMin, 0)); //minimum rozsahu
    var r_max = parseFloat(that.check($o.rangeMax, 100)); //maximum rozsahu
    var tick_step = that.check($o.tickStep, 5); //krok maleho tiku
    var main_tick_step = that.check($o.mainTickStep, 10); //krok hlavniho tiku s oznacenim
    var decimals = that.check($o.decimals || $o.digitalPrecision, 2); //pocet desetinnych mist pro zaokrouhleni digitalni hodnoty
    var o_units = $o.units || " "; //jednotky
    var colorZones = $o.colorZones || null; //barevne zony
    var colorOffLimits = $o.colorOffLimits || "#ff7400";
    var levelColor1 = $o.levelColor1 || "#01d2ff";
    var levelColor2 = $o.levelColor2 || "#001070";
    var fSize = 28;

    // Get SVG elements for manipulation
    var bargraph_area = that.getChildByTag("bargraph_area"),
        //cely objekt
    bargraph_level = that.getChildByTag("bargraph_level"),
        //hladina bargrafu
    bargraph_capacity = that.getChildByTag("bargraph_capacity"),
        //celkova velikost (kapacita) bargrafu
    border = that.getChildByTag("border"),
        //okraj (ramecek)
    digitalValue = that.getChildByTag("digitalval"),
        //digitalni hodnota
    textBox = that.getChildByTag("text_box") || that.getChildByTag("display_box"),
        //ramecek digitalni hodnoty
    stopC1 = that.getChildByTag("stopC1"),
        stopC2 = that.getChildByTag("stopC2"),
        units = that.getChildByTag("units"); //jednotky

    // Deprecated warning
    if (!textBox) {
        that.log.warn(that.id + ': Please upgrade this component');
    }

    //Global variables
    var center_x = bargraph_area.getBBox().width / 2; //x-ova souradnice stredu 
    var center_y = bargraph_area.getBBox().height / 2; //y-ova souradnice stredu
    var centerXDBox = textBox.getBBox().x + textBox.getBBox().width / 2;
    var centerYDBox = textBox.getBBox().y + textBox.getBBox().height / 2;
    var font_size_units = 24;
    var font_size_digitalval = 24;
    var tick_counter = 0;
    var tick_height;
    var tick_width;
    var labels = new Array();
    var colorRanges = new Array();
    var zoneCounter = 0;

    //Set level color
    stopC1.style.stopColor = levelColor1;
    stopC2.style.stopColor = levelColor2;

    //Set units
    units.textContent = o_units;
    units.setAttributeNS(null, "style", "font-size:" + font_size_units + "px; fill:#ffffff; font-family:Arial");
    units.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(centerXDBox - units.parentNode.getBBox().width / 2 - units.parentNode.getBBox().x) + "," + 0 + ")");

    //Set digital value
    digitalValue.setAttributeNS(null, "style", "font-size:" + font_size_digitalval + "px; font-family:Arial");

    // Draw ticks
    //tick_counter = (Math.abs(1000000 * r_max - 1000000 * r_min) / 1000000 / tick_step);
    tick_counter = Math.abs(r_max - r_min) / tick_step;
    tick_height = 1.2;
    tick_width = 20;
    var i = 0;
    while (i <= tick_counter + 0.1) {
        createTick(i, tick_height, tick_width);
        i = i + 1;
    }

    //Draw main ticks
    //var tick_counter = (Math.abs(1000000 * r_max - 1000000 * r_min) / 1000000 / main_tick_step);
    //alert(Math.abs(1000000 * r_max - 1000000 * r_min) / 1000000 / main_tick_step);
    tick_counter = Math.abs(r_max - r_min) / main_tick_step;
    tick_height = 1.6;
    tick_width = 40;
    var i = 0;
    while (i <= tick_counter + 0.1) {
        createTick(i, tick_height, tick_width);
        createLabel(i);
        i = i + 1;
    }
    //Draw color range
    for (var n = 0; n < colorZones.length; n++) {
        drawColorRange(parseFloat(colorZones[n].startValue), parseFloat(colorZones[n].endValue), colorZones[n].color);
    }

    // Add anonymous function as event listener. There are two events
    // 'read' - it is called every time when item is read
    // 'change' - called for the first time and every time item value is changed    
    that.$c.value.on('change', function (itm) {
        var level = itm.getValue();
        if (level >= r_min && level <= r_max) {
            bargraph_level.setAttributeNS(null, "height", bargraph_capacity.getBBox().height * (level - r_min) / Math.abs(r_max - r_min));
            border.style.fill = "#000000";
            digitalValue.style.fill = "#00ffff";
        } else {
            if (level > r_max) {
                bargraph_level.setAttributeNS(null, "height", bargraph_capacity.getBBox().height);
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                while (r_max <= level) {
                    var tmp = r_min;
                    r_min = r_min + 0.5 * Math.abs(r_max - tmp);
                    r_max = r_max + 0.5 * Math.abs(r_max - tmp);
                }
                changeLabels();
                changeColorRange();
                bargraph_level.setAttributeNS(null, "height", (bargraph_capacity.getBBox().height) * (level - r_min) / Math.abs(r_max - r_min));
                digitalval.style.fill = "#00ffff";
                */
            } else {
                bargraph_level.setAttributeNS(null, "height", 0.001);
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                while (r_min >= level) {
                    var tmp = r_min;
                    r_min = r_min - Math.abs(r_max - tmp);
                    r_max = r_max - Math.abs(r_max - tmp);
                }
                changeLabels();
                changeColorRange();
                bargraph_level.setAttributeNS(null, "height", (bargraph_capacity.getBBox().height) * (level - r_min) / Math.abs(r_max - r_min));
                digitalval.style.fill = "#00ffff";
                */
            }
        }
        digitalValue.textContent = level.toFixed(decimals);
        transformDisplay();
    });

    function createTick(i, tick_height, tick_width) {
        var x = bargraph_capacity.getBBox().x - tick_width;
        var y = bargraph_capacity.getBBox().y + bargraph_capacity.getBBox().height - i * bargraph_capacity.getBBox().height / tick_counter - tick_height / 2;
        var elem = document.createElementNS("http://www.w3.org/2000/svg", "rect");
        elem.setAttributeNS(null, "x", x);
        elem.setAttributeNS(null, "y", y);
        elem.setAttributeNS(null, "width", tick_width);
        elem.setAttributeNS(null, "height", tick_height);
        elem.setAttributeNS(null, "style", "fill:#ffffff");
        bargraph_area.appendChild(elem);
    }

    function createLabel(i) {
        var x = bargraph_capacity.getBBox().x - tick_width;
        var y = bargraph_capacity.getBBox().y + bargraph_capacity.getBBox().height - i * bargraph_capacity.getBBox().height / tick_counter - tick_height / 2;
        var font_size = 24;
        var translate_x;
        var translate_y;
        var text = document.createElementNS("http://www.w3.org/2000/svg", "text");
        text.setAttributeNS(null, "x", x);
        text.setAttributeNS(null, "y", y);
        text.setAttributeNS(null, "fill", "#ffffff");
        text.setAttributeNS(null, "style", "font-size:" + font_size + "px; font-family:Arial");
        text.textContent = Math.round((parseFloat(r_min) + i * main_tick_step) * 10000) / 10000;
        bargraph_area.appendChild(text);
        translate_x = -text.getBBox().width - 3;
        translate_y = text.getBBox().height / 2 - 3;
        text.setAttributeNS(null, "transform", "translate(" + translate_x + "," + translate_y + ")");
        labels[i] = text;
    }

    function changeLabels() {
        var translate_x;
        var translate_y;
        for (var i = 0; i < labels.length; i++) {
            labels[i].textContent = Math.round((parseFloat(r_min) + i * parseFloat(main_tick_step)) * 100) / 100;
            translate_x = -labels[i].getBBox().width - 3;
            translate_y = labels[i].getBBox().height / 2 - 3;
            labels[i].setAttributeNS(null, "transform", "translate(" + translate_x + "," + translate_y + ")");
        }
    }

    function drawColorRange(startValue, endValue, color) {
        var start = startValue;
        var end = endValue;
        if (start < r_min) start = r_min;
        if (end > r_max) end = r_max;
        var startHeight = bargraph_capacity.getBBox().height * (start - r_min) / Math.abs(r_max - r_min);
        var endHeight = bargraph_capacity.getBBox().height * (end - r_min) / Math.abs(r_max - r_min);
        var x = bargraph_capacity.getBBox().x + bargraph_capacity.getBBox().width + 1;
        var y = bargraph_capacity.getBBox().y + bargraph_capacity.getBBox().height - endHeight;

        var rect = document.createElementNS("http://www.w3.org/2000/svg", "rect");
        rect.setAttributeNS(null, "style", "fill:" + color + "; stroke:none");
        rect.setAttributeNS(null, "x", x);
        rect.setAttributeNS(null, "y", y);
        rect.setAttributeNS(null, "height", endHeight - startHeight);
        rect.setAttributeNS(null, "width", 8);
        bargraph_area.appendChild(rect);
        colorRanges[zoneCounter] = new colorRangeObject(rect, startValue, endValue, color);
        zoneCounter++;
    }

    function changeColorRange() {
        for (var j = 0; j < colorRanges.length; j++) {
            var start = colorRanges[j].start;
            var end = colorRanges[j].end;
            if (start < parseFloat(r_min)) start = parseFloat(r_min);
            if (end > r_max) end = r_max;
            var startHeight = bargraph_capacity.getBBox().height * (start - parseFloat(r_min)) / Math.abs(parseFloat(r_max) - parseFloat(r_min));
            var endHeight = bargraph_capacity.getBBox().height * (end - parseFloat(r_min)) / Math.abs(parseFloat(r_max) - parseFloat(r_min));
            var x = bargraph_capacity.getBBox().x + bargraph_capacity.getBBox().width + 1;
            var y = bargraph_capacity.getBBox().y + bargraph_capacity.getBBox().height - endHeight;
            colorRanges[j].cRObject.setAttributeNS(null, "x", x);
            colorRanges[j].cRObject.setAttributeNS(null, "y", y);
            colorRanges[j].cRObject.setAttributeNS(null, "height", endHeight - startHeight);
        }
    }

    function colorRangeObject(cRObject, start, end, color) {
        this.cRObject = cRObject;
        this.start = start;
        this.end = end;
        this.color = color;
    }

    function transformDisplay() {
        if (textBox) {
            var fontSize = digitalValue.style.fontSize || digitalValue.parentNode.style.fontSize;
            if (digitalValue.parentNode.getBBox().width >= textBox.getBBox().width * 0.95) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.90 + "px";
            } else if (digitalValue.parentNode.getBBox().width < textBox.getBBox().width * 0.80 && parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 < fSize) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 + "px";
            }

            digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(textBox.getBBox().x + (textBox.getBBox().width / 2 - digitalValue.parentNode.getBBox().width / 2) - digitalValue.parentNode.getBBox().x) + "," + parseInt(textBox.getBBox().y + (textBox.getBBox().height / 2 - digitalValue.parentNode.getBBox().height / 2) - digitalValue.parentNode.getBBox().y) + ")");
        }
    }

    return that;
};
'use strict';

/**
 * SVG component represents Button.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT} 
 * @returns {REX.UI.SVG.Fan} New SVG Button component
 */
REX.UI.SVG.Button = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.HTMLComponent(svgElem, args);
    var $o = that.options || {};
    var onMouseDownValue = that.parseBoolean($o.reverseMeaning) ? 0 : 1;
    var fontScale = parseFloat(that.check($o.fontScale, 1));
    var elementTitle = $(that.element).find('title').text();

    //var textElems = that.element.getElementsByTagName('text');
    var button = $(document.createElement('button')).button();
    $(that.div).append(button);

    button.attr('style', 'width:100%; height:100%; white-space: normal;');
    button.text($o.text);

    // value_R and value_W remains for backward compatibility
    that.$c.refresh_from = that.$c.refresh_from || that.$c.BUTTON_R || that.$c.BUTTON;

    var oldDisable = that.disable;
    that.disable = function () {
        oldDisable.apply(that, arguments);
        button.attr("disabled", true);
        button.css('opacity', 0.5);
        button.removeClass('ui-state-active', 'ui-state-hover');
        button.addClass('ui-state-default');
    };

    var oldEnable = that.enable;
    that.enable = function () {
        oldEnable.apply(that, arguments);
        button.removeAttr("disabled");
        button.css('opacity', '');
    };

    // Init font autoresize+
    function updateFontSize() {
        var ctm = that.svg.getScreenCTM();
        // Scale according the width or height which is better
        button.css('font-size', Math.min(ctm.a, ctm.d) * fontScale + 'em');
    }
    updateFontSize();
    $(window).resize(function () {
        updateFontSize();
    });

    if (that.$c.BUTTON.type === 'R') {
        that.log.error('Connection String: ' + that.$c.BUTTON.cstring + '(' + that.$c.BUTTON.alias + ') is read-only');
        return that;
    }

    var active = false;
    if ($o.type === 'ToggleButton') {

        //four lines to get background for state-active class    
        var $inspector = $("<div>").css('display', 'none').addClass('ui-state-active');
        $("body").append($inspector); // add to DOM, in order to read the CSS property
        var background = $inspector.css('background-color');
        $inspector.remove(); // and remove from DOM

        that.$c.refresh_from.on('change', function (itm) {
            if (itm.getValue() === 1 - onMouseDownValue) {
                $(button).css("background", "");
                active = false;
            } else {
                $(button).css("background", background);
                active = true;
            }
        });
    }

    var pressed = false;

    button.bind('touchend touchcancel touchleave mouseup mouseout', function (evt) {
        evt.preventDefault();
        this.blur();
        if (evt.handled !== true) {
            // Primary mouse button only       
            // Invoke only when the button was pressed before
            if (!(evt.button && evt.button > 0) && pressed) {
                that.log.debug($o.text + ' ' + evt.type);
                if ($o.type === 'PushButton') {
                    that.$c.BUTTON.write(1 - onMouseDownValue).catch(function (err) {
                        that.log.error("Write failed", err);
                    });
                }
                that.emit('mouseup');
                pressed = false;
            }
            evt.handled = true;
        } else {
            return false;
        }
    }).bind('touchstart mousedown', function (evt) {
        evt.preventDefault();
        this.focus();
        if (evt.handled !== true) {
            // Primary mouse button only            
            if (!(evt.button && evt.button > 0)) {
                if ($o.type === 'ToggleButton') {
                    if (!active) {
                        that.$c.BUTTON.write(onMouseDownValue).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        active = true;
                    } else {
                        that.$c.BUTTON.write(1 - onMouseDownValue).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        active = false;
                    }
                } else {
                    that.log.debug($o.text + ' ' + evt.type);
                    that.$c.BUTTON.write(onMouseDownValue).catch(function (err) {
                        that.log.error("Write failed", err);
                    });
                }
                that.emit('mousedown');
                pressed = true;
            }
            evt.handled = true;
        } else {
            return false;
        }
    });

    that.disable();
    return that;
};
'use strict';

/**
 * SVG component represents ComboBox.
 * @param {SVGElement} svgElem
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Fan} New SVG ComboBox component
 */
REX.UI.SVG.ComboBox = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.HTMLComponent(svgElem, args);
    var $o = that.options || {};
    var values = $o.values;
    var showValue = that.parseBoolean($o.showValue);
    var fontScale = that.checkNumber($o.fontScale, 1);
    var valueType = that.check($o.valueType, 'number').toLowerCase();

    that.$c.refresh_from = that.$c.refresh_from || that.$c.read || that.$c.value || that.$c.write;
    if (!that.$c.refresh_from) {
        that.log.error('Refresh from connection string is not defined! Cannot continue!');
        return that;
    }

    var selectorId = 'selector' + that.element.id;
    var select = that.select = $('<select name="' + selectorId + '" id="' + selectorId + '">');
    $(that.div).append(select);
    for (var i = 0; i < values.length; i++) {
        var desc = '';
        if (!values[i].desc) {
            desc = values[i].value;
        } else {
            desc = showValue ? values[i].value + ': ' + values[i].desc : values[i].desc;
        }
        select.append('<option value="' + values[i].value + '">' + desc + '</option>');
    }
    select.selectmenu();

    select.on('selectmenuchange', function (event, ui) {
        var val = $(this).val();
        if (valueType === 'number') {
            val = parseInt(val);
        }
        that.$c.value.write(val).catch(function (err) {
            that.log.error("Write failed", err);
        });
    });

    var selectBtn = $(that.div).find('.ui-selectmenu-button');
    var selectIcon = $(that.div).find('.ui-selectmenu-icon');
    var selectMenu = $('#' + selectorId + '-menu');
    selectBtn.css({ "padding-top": "0px", "padding-bottom": "0px" });

    var firstRefresh = true;
    that.$c.refresh_from.on('change', function (itm) {
        var value = itm.getValue();
        select.val(value.toString());
        if (select.find('option[value="' + value + '"]').length == 0) {
            select.append('<option value="' + value + '">' + value + '</option>');
            select.selectmenu();
            select.val(value.toString());
            select.selectmenu("refresh");
            firstRefresh = true;
        } else {
            select.selectmenu("refresh");
        }
        if (firstRefresh) {
            that.updatePosition();
            firstRefresh = false;
        }
    });

    var oldDisable = that.disable;
    that.disable = function () {
        "use strict";

        oldDisable.apply(that, arguments);
        select.selectmenu('disable');
    };

    var oldEnable = that.enable;
    that.enable = function () {
        "use strict";

        if (oldEnable.apply(that, arguments)) {
            select.selectmenu('enable');
            return true;
        }
    };

    var oldUpdatePosition = that.updatePosition;
    that.updatePosition = function () {
        oldUpdatePosition.apply(that, arguments);
        updateFontSize();
        select.selectmenu("option", "width", $(that.div).width());
        selectBtn.css({ 'line-height': that.div.style.height, 'height': that.div.style.height });
        selectIcon.css('margin-top', 'calc((' + that.div.style.height + ' - 16px )/ 2');
    };

    // Init font autoresize
    function updateFontSize() {
        var ctm = that.svg.getScreenCTM();
        // Scale according the width or height which is better
        selectBtn.css('font-size', Math.min(ctm.a, ctm.d) * fontScale + 'em');
        selectMenu.css('font-size', Math.min(ctm.a, ctm.d) * fontScale + 'em');
    }

    that.disable();
    that.updatePosition();
    return that;
};
"use strict";

/**
 * SVG component represents control led
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT} 
 * @returns {REX.UI.SVG.ControlLed} New Control Led component
 */
REX.UI.SVG.ControlLed = function (svgElem, args) {
  // Inherit from base component
  var that = new REX.UI.SVG.Component(svgElem, args);
  // Store options for simple usage
  var $o = that.options || {};

  // Load options or default values
  var onColor = that.parseBoolean($o.reverseMeaning) ? "white" : $o.color;
  var offColor = that.parseBoolean($o.reverseMeaning) ? $o.color : "white";

  // Get SVG elements for manipulation
  var oled1 = that.getChildByTag('radialgradient-start');
  var oled2 = that.getChildByTag('radialgradient-stop');
  var radialGradientId = that.getChildByTag('radialgradient').id;
  var path = that.getChildByTag('path');

  path.style.fill = "url(#" + radialGradientId + ")";

  // Add anonymous function as event listener. There are two events
  // 'read' - it is called every time when item is read
  // 'change' - called for the first time and every time item value is changed
  that.$c.LIGHT.on('change', function (i) {
    if (i.getValue()) {
      oled1.style.stopColor = onColor;
      oled2.style.stopColor = onColor;
    } else {
      oled1.style.stopColor = offColor;
      oled2.style.stopColor = offColor;
    }
  });

  return that;
};
"use strict";

REX.UI.SVG.CustomHTML = function (svgElem, args) {
     // Inherit from base component
     var that = new REX.UI.SVG.HTMLComponent(svgElem, args);
     var $o = that.options || {};
     var html = $o.html;

     $(that.div).html(html);

     return that;
};
'use strict';

/**
 * SVG component represents Display.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Fan} New SVG Display component
 */
REX.UI.SVG.Display = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var range_min = that.checkNumber($o.rangeMin, 0);
    var range_max = that.checkNumber($o.rangeMax, 100);
    var color_max = $o.colorAbove || '#ff0000';
    var color_min = $o.colorBelow || '#ffff00';
    var color = $o.color || "black";
    var format = $o.format || '';
    var text_format = $o.text_format || '';
    var scale = that.checkNumber($o.scale, 1);
    var offset = that.checkNumber($o.offset, 0);
    var decimals = that.checkNumber($o.decimals, 4);
    var units = $o.units || '';

    // Get SVG elements for manipulation
    var display = that.getChildByTag("display");

    that.$c.value.on('change', function (itm) {
        switch (format.toLowerCase()) {
            case 'text':
                if (text_format) {
                    if (text_format.indexOf("hh:mm") !== -1) {
                        display.textContent = that.time2str(itm.getValue(), text_format);
                    }
                }
                // TODO: Add more text formats
                else {
                        display.textContent = itm.getValue();
                    }
                break;
            case 'date':
                display.textContent = that.date2str(that.getDateFromREXSeconds(itm.getValue()));
                break;
            case 'time':
                display.textContent = that.getDateFromREXSeconds(itm.getValue()).toLocaleTimeString();
                break;
            case 'datetime':
                display.textContent = that.date2str(that.getDateFromREXSeconds(itm.value)) + ' ' + that.getDateFromREXSeconds(itm.value).toLocaleTimeString();
                break;
            default:
                if (itm.getValue() < range_min) {
                    display.style.fill = color_min;
                } else if (itm.getValue() > range_max) {
                    display.style.fill = color_max;
                } else {
                    display.style.fill = color;
                }
                var resultValue = itm.value * scale + offset;
                display.textContent = '' + resultValue.toFixed(decimals);
                break;
        }
        if (units) {
            display.textContent = display.textContent + ' ' + units;
        }
    });

    return that;
};
'use strict';

/**
 * SVG component represents DisplayWithBox.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.DigitalValue} New SVG DigitalValue component
 */

REX.UI.SVG.DisplayWithBox = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Get options or default values
    var decimals = that.checkNumber($o.decimals, 2),
        rangeMin = that.checkNumber($o.rangeMin, 0),
        rangeMax = that.checkNumber($o.rangeMax, 100),
        colorAbove = $o.colorAbove || '#ff0000',
        colorBelow = $o.colorBelow || '#ffff00',
        color = $o.color || "#00ffff",
        o_units = $o.units || " ";
    var format = $o.format || '';
    var scale = that.checkNumber($o.scale, 1);
    var offset = that.checkNumber($o.offset, 0);

    // Get SVG elements for manipulation
    var digitalvalue_area = that.getChildByTag("digitalval_area"),
        digitalValue = that.getChildByTag("digitalval"),
        textBox = that.getChildByTag("textbox"),
        text = that.getChildByTag("text"),
        units = that.getChildByTag("units");

    //Global variables
    var center_x = digitalvalue_area.getBBox().width / 2,
        center_y = digitalvalue_area.getBBox().height / 2;

    //Set units
    units.textContent = o_units;
    units.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(center_x - units.parentNode.getBBox().width / 2 - units.parentNode.getBBox().x) + "," + 0 + ")");

    function resizeFont() {
        var fontSize = digitalValue.style.fontSize,
            dvBB = digitalValue.parentNode.getBBox(),
            tbBB = textBox.getBBox();

        // Change font size        
        if (dvBB.width >= tbBB.width * 0.95) {
            digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.9 + "px";
        } else if (dvBB.width < tbBB.width * 0.9 && dvBB.height < tbBB.height) {
            digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.1 + "px";
        }
        // Center text
        digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(digitalvalue_area.getBBox().width / 2 - dvBB.width / 2 - dvBB.x) + "," + 0 + ")");
    }

    that.$c.value.on('change', function (itm) {
        switch (format.toLowerCase()) {
            case 'date':
                digitalValue.textContent = that.date2str(that.getDateFromREXSeconds(itm.value));
                break;
            case 'time':
                digitalValue.textContent = that.getDateFromREXSeconds(itm.value).toLocaleTimeString();
                break;
            case 'datetime':
                digitalValue.textContent = that.date2str(that.getDateFromREXSeconds(itm.value)) + ' ' + that.getDateFromREXSeconds(itm.value).toLocaleTimeString();
                break;
            default:
                if (itm.getValue() < rangeMin) {
                    digitalValue.style.fill = colorBelow;
                } else if (itm.getValue() <= rangeMax) {
                    digitalValue.style.fill = color;
                } else {
                    digitalValue.style.fill = colorAbove;
                }
                var resultValue = itm.getValue() * scale + offset;
                digitalValue.textContent = resultValue.toFixed(decimals);
                break;
        }
        resizeFont();
    });

    resizeFont();

    return that;
};
"use strict";

/**
 * SVG component represents Gauge.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Gauge} New SVG Gauge component
 */
REX.UI.SVG.Gauge180 = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var r_min = parseFloat(that.check($o.rangeMin, 0)); //minimum rozsahu
    var r_max = parseFloat(that.check($o.rangeMax, 100)); //maximum rozsahu
    var tick_step = that.check($o.tickStep, 5); //krok maleho tiku
    //krok hlavniho tiku s oznacenim
    var main_tick_step = that.check($o.mainTickStep, 10);
    //pocet desetinnych mist pro zaokrouhleni digitalni hodnoty
    var decimals = that.check($o.decimals || $o.digitalPrecision, 2);
    var o_units = $o.units || " "; //jednotky
    var colorZones = $o.colorZones || null; //barevne rozsahy
    var colorOffLimits = $o.colorOffLimits || "#ff7400";
    var fSize = 28;

    // Get SVG elements for manipulation
    var gauge_area = that.getChildByTag("gauge_area"); //cely objekt
    var hand = that.getChildByTag("hand"); //rucicka
    var middle_circle = that.getChildByTag("middle"); //kruhovy stred
    var border = that.getChildByTag("border"); //okraj
    var tick_0 = that.getChildByTag("tick_0"); //maly tik v pocatku
    var main_tick_0 = that.getChildByTag("main_tick_0"); //hlavni oznaceny tik v pocatku
    var digitalValue = that.getChildByTag("digitalval"); //digitalni hodnota
    var units = that.getChildByTag("units"); //jednotky
    var textBox = that.getChildByTag("text_box"); // prostor pro vykresleni hodnoty
    // Deprecated warning
    if (!textBox) {
        that.log.warn(that.id + ': Please upgrade this component');
    }

    //Global variables
    var center_x = gauge_area.getBBox().width / 2; //x-ova souradnice stredu 
    var center_y = gauge_area.getBBox().height - middle_circle.getBBox().height / 2 - 1; //y-ova souradnice stredu;
    var font_digitalval = 24;
    var main_tick_size = 5;
    var main_tick_color = "#ffffff";
    var tick_counter; //pocet malych tiku
    var main_tick_counter; //pocet hlavnich oznacenych tiku                          
    var tick_angle; //uhel mezi jednotlivymi tiky
    var labels = new Array(); //pole hodnot pro popis osy

    //Fill color, opacity, size
    tick_0.setAttributeNS(null, "style", "fill:#ffffff");
    main_tick_0.setAttributeNS(null, "style", "fill-opacity:0");
    tick_0.setAttributeNS(null, "height", tick_0.getBBox().height / 2);
    tick_0.setAttributeNS(null, "y", tick_0.getBBox().y + tick_0.getBBox().height * 2 / 2 - tick_0.getBBox().height / 2);
    hand.setAttributeNS(null, "style", "fill-opacity:1");

    //Set units
    units.textContent = o_units;
    units.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(center_x - units.parentNode.getBBox().width / 2 - units.parentNode.getBBox().x) + "," + 10 + ")");

    //Draw ticks
    tick_counter = Math.abs(r_max - r_min) / tick_step;
    tick_angle = 180 / tick_counter;
    var i = 0;
    while (i <= tick_counter) {
        createTick(i, "#" + tick_0.id);
        i = i + 1;
    }
    //Draw main ticks
    main_tick_counter = Math.abs(r_max - r_min) / main_tick_step;
    tick_angle = 180 / main_tick_counter;
    i = 0;
    while (i <= main_tick_counter) {
        //createTick(i, "#main_tick_0");
        createLabel(i);
        createMainTick(i);
        i = i + 1;
    }

    //Draw color range
    for (var n = 0; n < colorZones.length; n++) {
        drawColorRange(parseFloat(colorZones[n].startValue), parseFloat(colorZones[n].endValue), colorZones[n].color);
    }

    // Change z-index on the top
    hand.parentNode.appendChild(hand); //posunuti rucicky v hierarchii uplne nahoru
    middle_circle.parentNode.appendChild(middle_circle); //posunuti kruhoveho stredu v hierarchii uplne nahoru
    digitalValue.setAttributeNS(null, "style", "font-size:" + font_digitalval + "px");

    // Add anonymous function as event listener. There are two events
    // 'read' - it is called every time when item is read
    // 'change' - called for the first time and every time item value is changed    
    that.$c.value.on('change', function (itm) {
        var value = itm.getValue();
        var angle = (value - r_min) * (180 / Math.abs(r_max - r_min)) - 90;
        if (value >= r_min && value <= r_max) {
            hand.setAttributeNS(null, "transform", "rotate(" + angle + "," + center_x + "," + center_y + ")");
            digitalValue.style.fill = "#00ffff";
            border.setAttributeNS(null, "style", "fill:#000000");
        } else {
            if (value > r_max) {

                hand.setAttributeNS(null, "transform", "rotate(" + 90 + "," + center_x + "," + center_y + ")");
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                var tmp = r_min;
                r_min = r_min + 0.5 * Math.abs(r_max - tmp);
                r_max = r_max + 0.5 * Math.abs(r_max - tmp);
                changeLabels();
                */
            } else {

                hand.setAttributeNS(null, "transform", "rotate(" + -90 + "," + center_x + "," + center_y + ")");
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                var tmp = r_min;
                r_min = r_min - 0.5 * Math.abs(r_max - tmp);
                r_max = r_max - 0.5 * Math.abs(r_max - tmp);
                changeLabels();
                */
            }
        }
        digitalValue.innerHTML = value.toFixed(decimals);
        transformDisplay();
    });

    function createTick(i, tick_type) {
        var mat_a = Math.cos(tick_angle * i * Math.PI / 180);
        var mat_b = Math.sin(tick_angle * i * Math.PI / 180);
        var mat_e = -center_x * Math.cos(tick_angle * i * Math.PI / 180) + center_y * Math.sin(tick_angle * i * Math.PI / 180) + center_x;
        var mat_f = -center_x * Math.sin(tick_angle * i * Math.PI / 180) - center_y * Math.cos(tick_angle * i * Math.PI / 180) + center_y;

        var elem = document.createElementNS("http://www.w3.org/2000/svg", "use");
        elem.setAttributeNS("http://www.w3.org/1999/xlink", "href", tick_type);
        elem.setAttributeNS(null, "transform", "matrix(" + mat_a + "," + mat_b + "," + -mat_b + "," + mat_a + "," + mat_e + "," + mat_f + ")");
        gauge_area.appendChild(elem);
    }

    function createMainTick(i) {
        var x = center_x + Math.sqrt(center_x / 1.888 * center_x / 1.888 + center_y / 1.888 * center_y / 1.888) * Math.cos((180 - tick_angle * i) * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 1.888 * center_x / 1.888 + center_y / 1.888 * center_y / 1.888) * Math.sin((180 - tick_angle * i) * Math.PI / 180);

        var circle = document.createElementNS("http://www.w3.org/2000/svg", "circle");
        circle.setAttributeNS(null, "cx", x);
        circle.setAttributeNS(null, "cy", y);
        circle.setAttributeNS(null, "r", main_tick_size);
        circle.setAttributeNS(null, "fill", main_tick_color);
        circle.setAttributeNS(null, "style", "stroke:none");
        gauge_area.appendChild(circle);
    }

    function createLabel(i) {
        var x = center_x + Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.cos((180 - tick_angle * i) * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.sin((180 - tick_angle * i) * Math.PI / 180);

        var font_size = 22;
        if (i > 0 && i < main_tick_counter) {
            y = y + font_size / 2 + 1;
        }
        var text = document.createElementNS("http://www.w3.org/2000/svg", "text");
        text.setAttributeNS(null, "x", x);
        text.setAttributeNS(null, "y", y);
        text.setAttributeNS(null, "fill", "#ffffff");
        text.setAttributeNS(null, "style", "font-size:" + font_size + "px; font-family:Arial");
        text.textContent = Math.round((parseFloat(r_min) + i * main_tick_step) * 100) / 100;
        gauge_area.appendChild(text);
        if (i > main_tick_counter / 2) {
            var translate_x = -text.getBBox().width - 1;
            text.setAttributeNS(null, "transform", "translate(" + translate_x + "," + 0 + ")");
        }
        if (i == main_tick_counter / 2) {
            var translate_x = -text.getBBox().width / 2;
            var translate_y = text.getBBox().height / 5;
            text.setAttributeNS(null, "transform", "translate(" + translate_x + "," + translate_y + ")");
        }
        labels[i] = text;
    }

    function changeLabels() {
        for (var i = 0; i < labels.length; i++) {
            labels[i].textContent = Math.round((r_min + i * main_tick_step) * 100) / 100;
        }
    }

    function drawColorRange(startValue, endValue, color) {
        var path = document.createElementNS("http://www.w3.org/2000/svg", "path");
        var x = center_x + Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.cos(180 * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.sin(180 * Math.PI / 180);
        path.setAttributeNS(null, "style", "fill:none; stroke:" + color + "; stroke-width:8; stroke-opacity:0.8");
        path.setAttributeNS(null, "d", "M " + x + " " + y);
        gauge_area.appendChild(path);

        if (startValue < r_min) startValue = r_min;
        if (endValue > r_max) endValue = r_max;

        var startAngle = (startValue - r_min) * (180 / Math.abs(r_max - r_min));
        var endAngle = (endValue - r_min) * (180 / Math.abs(r_max - r_min));
        var i = 0;
        while (startAngle <= endAngle) {
            var radians = (180 - startAngle) / 180 * Math.PI;
            var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 4 * center_x / 4 + center_y / 4 * center_y / 4);
            var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 4 * center_x / 4 + center_y / 4 * center_y / 4);
            //var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 1.7 * center_x / 1.7 + center_y / 1.7 * center_y / 1.7);
            //var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 1.7 * center_x / 1.7 + center_y / 1.7 * center_y / 1.7);
            //var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 1.46 * center_x / 1.46 + center_y / 1.46 * center_y / 1.46);
            //var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 1.46 * center_x / 1.46 + center_y / 1.46 * center_y / 1.46);
            var e = path.getAttribute("d");
            if (i == 0) {
                var d = e + " M " + px + " " + py;
            } else {
                var d = e + " L " + px + " " + py;
            }
            path.setAttributeNS(null, "d", d);
            startAngle += 0.5;
            i++;
        }
    }

    function transformDisplay() {
        if (textBox) {
            var fontSize = digitalValue.style.fontSize || digitalValue.parentNode.style.fontSize;
            if (digitalValue.parentNode.getBBox().width >= textBox.getBBox().width * 0.95) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.90 + "px";
            } else if (digitalValue.parentNode.getBBox().width < textBox.getBBox().width * 0.80 && parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 < fSize) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 + "px";
            }

            digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(textBox.getBBox().x + (textBox.getBBox().width / 2 - digitalValue.parentNode.getBBox().width / 2) - digitalValue.parentNode.getBBox().x) + "," + parseInt(textBox.getBBox().y + (textBox.getBBox().height / 2 - digitalValue.parentNode.getBBox().height / 2) - digitalValue.parentNode.getBBox().y) + ")");
        }
    }

    return that;
};
"use strict";

/**
 * SVG component represents Gauge.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Gauge} New SVG Gauge component
 */
REX.UI.SVG.Gauge270 = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var r_min = parseFloat(that.check($o.rangeMin, 0)); //minimum rozsahu
    var r_max = parseFloat(that.check($o.rangeMax, 100)); //maximum rozsahu
    var tick_step = that.check($o.tickStep, 5); //krok maleho tiku
    var main_tick_step = that.check($o.mainTickStep, 10); //krok hlavniho tiku s oznacenim
    var decimals = that.check($o.decimals || $o.digitalPrecision, 2); //pocet desetinnych mist pro zaokrouhleni digitalni hodnoty
    var o_units = $o.units || " "; //jednotky
    var colorZones = $o.colorZones || null; //barevne rozsahy
    var colorOffLimits = $o.colorOffLimits || "#ff7400";
    var fSize = 28;

    // Get SVG elements for manipulation
    var gauge_area = that.getChildByTag("gauge_area"); //cely objekt
    var hand = that.getChildByTag("hand"); //rucicka
    var hand1 = that.getChildByTag("hand1"); //1.cast rucicky
    var hand2 = that.getChildByTag("hand2"); //2.cast rucicky
    var middle_circle = that.getChildByTag("middle"); //kruhovy stred
    var border = that.getChildByTag("border"); //okraj
    var tick_0 = that.getChildByTag("tick_0"); //maly tik v pocatku
    var main_tick_0 = that.getChildByTag("main_tick_0"); //hlavni oznaceny tik v pocatku
    var digitalValue = that.getChildByTag("digitalval"); //digitalni hodnota
    var units = that.getChildByTag("units"); //jednotky
    var textBox = that.getChildByTag("text_box"); // prostor pro vykresleni hodnoty
    // Deprecated warning
    if (!textBox) {
        that.log.warn(that.id + ': Please upgrade this component');
    }

    //Global variables
    var center_x = gauge_area.getBBox().width / 2; //x-ova souradnice stredu 
    var center_y = gauge_area.getBBox().height / 2; //y-ova souradnice stredu 
    var tick_counter; //pocet malych tiku
    var main_tick_counter; //pocet hlavnich oznacenych tiku 
    var main_tick_size = 5;
    var main_tick_color = "#ffffff";
    var tick_angle; //uhel mezi jednotlivymi tiky
    var labels = new Array(); //pole hodnot pro popis osy

    //Fill color, opacity, size
    tick_0.setAttributeNS(null, "style", "fill:#ffffff");
    main_tick_0.setAttributeNS(null, "style", "fill-opacity:0");
    tick_0.setAttributeNS(null, "height", tick_0.getBBox().height / 2);
    tick_0.setAttributeNS(null, "y", tick_0.getBBox().y + tick_0.getBBox().height * 2 / 2 - tick_0.getBBox().height / 2);
    hand.setAttributeNS(null, "style", "fill-opacity:1");

    //Set units
    units.textContent = '' + o_units;
    units.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(center_x - units.parentNode.getBBox().width / 2 - units.parentNode.getBBox().x) + "," + 0 + ")");

    //Draw ticks
    tick_counter = Math.abs(r_max - r_min) / tick_step;
    tick_angle = 270 / tick_counter;
    var i = 0;
    while (i <= tick_counter) {
        createTick(i, "#" + tick_0.id);
        i = i + 1;
    }
    //Draw main ticks
    main_tick_counter = Math.abs(r_max - r_min) / main_tick_step;
    tick_angle = 270 / main_tick_counter;
    i = 0;
    while (i <= main_tick_counter) {
        //createTick(i, "#main_tick_0");
        createMainTick(i);
        createLabel(i);
        i = i + 1;
    }

    //Draw color range
    for (var n = 0; n < colorZones.length; n++) {
        drawColorRange(parseFloat(colorZones[n].startValue), parseFloat(colorZones[n].endValue), colorZones[n].color);
    }

    // Change z-index on the top
    hand.parentNode.appendChild(hand); //posunuti rucicky v hierarchii uplne nahoru
    middle_circle.parentNode.appendChild(middle_circle); //posunuti kruhoveho stredu v hierarchii uplne nahoru

    // Add anonymous function as event listener. There are two events
    // 'read' - it is called every time when item is read
    // 'change' - called for the first time and every time item value is changed    
    that.$c.value.on('change', function (itm) {
        var value = itm.getValue();
        var angle = (value - r_min) * (270 / Math.abs(r_max - r_min));
        if (value >= r_min && value <= r_max) {
            hand1.setAttributeNS(null, "transform", "rotate(" + angle + "," + center_x + "," + center_y + ")");
            hand2.setAttributeNS(null, "transform", "rotate(" + angle + "," + center_x + "," + center_y + ")");
            digitalValue.style.fill = "#00ffff";
            border.setAttributeNS(null, "style", "fill:#000000");
        } else {
            if (value > r_max) {
                hand1.setAttributeNS(null, "transform", "rotate(" + 270 + "," + center_x + "," + center_y + ")");
                hand2.setAttributeNS(null, "transform", "rotate(" + 270 + "," + center_x + "," + center_y + ")");
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                var tmp = r_min;
                r_min = r_min + 0.5 * Math.abs(r_max - tmp);
                r_max = r_max + 0.5 * Math.abs(r_max - tmp);
                changeLabels();
                */
            } else {
                hand1.setAttributeNS(null, "transform", "rotate(" + 0 + "," + center_x + "," + center_y + ")");
                hand2.setAttributeNS(null, "transform", "rotate(" + 0 + "," + center_x + "," + center_y + ")");
                digitalValue.style.fill = colorOffLimits;
                border.style.fill = colorOffLimits;
                /*
                var tmp = r_min;
                r_min = r_min - 0.5 * Math.abs(r_max - tmp);
                r_max = r_max - 0.5 * Math.abs(r_max - tmp);
                changeLabels();
                */
            }
        }
        digitalValue.innerHTML = value.toFixed(decimals);
        transformDisplay();
    });

    function createTick(i, tick_type) {
        var mat_a = Math.cos(tick_angle * i * Math.PI / 180);
        var mat_b = Math.sin(tick_angle * i * Math.PI / 180);
        var mat_e = -center_x * Math.cos(tick_angle * i * Math.PI / 180) + center_y * Math.sin(tick_angle * i * Math.PI / 180) + center_x;
        var mat_f = -center_x * Math.sin(tick_angle * i * Math.PI / 180) - center_y * Math.cos(tick_angle * i * Math.PI / 180) + center_y;

        var elem = document.createElementNS("http://www.w3.org/2000/svg", "use");
        elem.setAttributeNS("http://www.w3.org/1999/xlink", "href", tick_type);
        elem.setAttributeNS(null, "transform", "matrix(" + mat_a + "," + mat_b + "," + -mat_b + "," + mat_a + "," + mat_e + "," + mat_f + ")");
        gauge_area.appendChild(elem);
    }

    function createMainTick(i) {
        var x = center_x + Math.sqrt(center_x / 1.888 * center_x / 1.888 + center_y / 1.888 * center_y / 1.888) * Math.cos((225 - tick_angle * i) * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 1.888 * center_x / 1.888 + center_y / 1.888 * center_y / 1.888) * Math.sin((225 - tick_angle * i) * Math.PI / 180);

        var circle = document.createElementNS("http://www.w3.org/2000/svg", "circle");
        circle.setAttributeNS(null, "cx", x);
        circle.setAttributeNS(null, "cy", y);
        circle.setAttributeNS(null, "r", main_tick_size);
        circle.setAttributeNS(null, "fill", main_tick_color);
        circle.setAttributeNS(null, "style", "stroke:none");
        gauge_area.appendChild(circle);
    }

    function createLabel(i) {
        var text = document.createElementNS("http://www.w3.org/2000/svg", "text");
        var x = center_x + Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.cos((225 - tick_angle * i) * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.sin((225 - tick_angle * i) * Math.PI / 180);
        var font_size = 28;
        if (i > 0 && i < main_tick_counter) {
            y = y + font_size / 2;
        }
        if (i == main_tick_counter / 2) {
            x = x - font_size / 3;
            y = y + font_size / 4;
        }
        if (i > main_tick_counter / 2) {
            x = x - font_size * 1.1;
        }
        text.setAttributeNS(null, "x", x);
        text.setAttributeNS(null, "y", y);
        text.setAttributeNS(null, "fill", "#ffffff");
        text.setAttributeNS(null, "style", "font-size:" + font_size + "px; font-family:Arial");
        text.textContent = Math.round((parseFloat(r_min) + i * main_tick_step) * 100) / 100;
        gauge_area.appendChild(text);
        labels[i] = text;
    }

    function changeLabels() {
        for (var i = 0; i < labels.length; i++) {
            labels[i].textContent = Math.round((parseFloat(r_min) + i * main_tick_step) * 100) / 100;
        }
    }

    function drawColorRange(startValue, endValue, color) {
        var path = document.createElementNS("http://www.w3.org/2000/svg", "path");
        var x = center_x + Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.cos(225 * Math.PI / 180);
        var y = center_y - Math.sqrt(center_x / 2 * center_x / 2 + center_y / 2 * center_y / 2) * Math.sin(225 * Math.PI / 180);
        path.setAttributeNS(null, "style", "fill:none; stroke:" + color + "; stroke-width:10; stroke-opacity:0.7");
        path.setAttributeNS(null, "d", "M " + x + " " + y);
        gauge_area.appendChild(path);

        if (startValue < r_min) startValue = r_min;
        if (endValue > r_max) endValue = r_max;

        var startAngle = (startValue - r_min) * (270 / Math.abs(r_max - r_min));
        var endAngle = (endValue - r_min) * (270 / Math.abs(r_max - r_min));
        var i = 0;
        while (startAngle <= endAngle) {
            var radians = (225 - startAngle) / 180 * Math.PI;
            //var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 3 * center_x / 3 + center_y / 3 * center_y / 3);
            //var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 3 * center_x / 3 + center_y / 3 * center_y / 3);
            var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 1.558 * center_x / 1.558 + center_y / 1.558 * center_y / 1.558);
            var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 1.558 * center_x / 1.558 + center_y / 1.558 * center_y / 1.558);
            //var px = center_x + Math.cos(radians) * Math.sqrt(center_x / 1.46 * center_x / 1.46 + center_y / 1.46 * center_y / 1.46);
            //var py = center_y - Math.sin(radians) * Math.sqrt(center_x / 1.46 * center_x / 1.46 + center_y / 1.46 * center_y / 1.46);
            var e = path.getAttribute("d");
            if (i == 0) {
                var d = e + " M " + px + " " + py;
            } else {
                var d = e + " L " + px + " " + py;
            }
            path.setAttributeNS(null, "d", d);
            startAngle += 0.5;
            i++;
        }
    }

    function transformDisplay() {
        if (textBox) {
            var fontSize = digitalValue.style.fontSize || digitalValue.parentNode.style.fontSize;
            if (digitalValue.parentNode.getBBox().width >= textBox.getBBox().width * 0.95) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.90 + "px";
            } else if (digitalValue.parentNode.getBBox().width < textBox.getBBox().width * 0.80 && parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 < fSize) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 + "px";
            }

            digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(textBox.getBBox().x + (textBox.getBBox().width / 2 - digitalValue.parentNode.getBBox().width / 2) - digitalValue.parentNode.getBBox().x) + "," + parseInt(textBox.getBBox().y + (textBox.getBBox().height / 2 - digitalValue.parentNode.getBBox().height / 2) - digitalValue.parentNode.getBBox().y) + ")");
        }
    }

    return that;
};
'use strict';

/**
 * SVG component represents General Component with multiple transformation
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT} 
 * @returns {REX.UI.SVG.GeneralComponent} New Genral Component component
 */
REX.UI.SVG.GeneralComponent = function (svgElem, args) {
    var that = new REX.UI.SVG.Component(svgElem, args);
    var $o = that.options || {};

    var bbox = that.element.getBBox(),
        ocmi = $o.colorMin || ' ',
        ocma = $o.colorMax || ' ',
        R_min = hexToR(ocmi) || 0,
        G_min = hexToG(ocmi) || 0,
        B_min = hexToB(ocmi) || 0,
        R_max = hexToR(ocma) || 0,
        G_max = hexToG(ocma) || 0,
        B_max = hexToB(ocma) || 0,
        rotationR = that.check($o.rotationRange, 0),
        rotationSMax = that.check($o.rotationSignalMax, 0),
        rotationSMin = that.check($o.rotationSignalMin, 0),
        rotOffsetX = parseFloat(that.element.getAttribute('inkscape:transform-center-x')) || 0,
        rotOffsetY = parseFloat(that.element.getAttribute('inkscape:transform-center-y')) || 0,
        elemCenterX = bbox.x + bbox.width / 2,
        elemCenterY = bbox.y + bbox.height / 2,
        scaleMax = that.check($o.scaleMax, 0),
        scaleMin = that.check($o.scaleMin, 0),
        scaleSMax = that.check($o.scaleSignalMax, 0),
        scaleSMin = that.check($o.scaleSignalMin, 0),
        scaleX = that.parseBoolean($o.scaleX),
        scaleY = that.parseBoolean($o.scaleY),
        opacityMax = that.check($o.opacityMax, 0),
        opacityMin = that.check($o.opacityMin, 0),
        opacityR = opacityMax - opacityMin || 0,
        opacitySMax = that.check($o.opacitySignalMax, 0),
        opacitySMin = that.check($o.opacitySignalMin, 0),
        txR = that.check($o.xRange, 0),
        txSMax = that.check($o.xSignalMax, 0),
        txSMin = that.check($o.xSignalMin, 0),
        tyR = that.check($o.yRange, 0),
        tySMax = that.check($o.ySignalMax, 0),
        tySMin = that.check($o.ySignalMin, 0);
    that.element.setAttribute('transform', '');

    that.changeColor = function (color) {
        if (REX.HELPERS.isNumber(color)) {
            var R, G, B, X;

            if (color > $o.colorSignalMin && color < $o.colorSignalMax) {
                R = Math.round(R_min + (R_max - R_min) * (color - $o.colorSignalMin) / ($o.colorSignalMax - $o.colorSignalMin));
                G = Math.round(G_min + (G_max - G_min) * (color - $o.colorSignalMin) / ($o.colorSignalMax - $o.colorSignalMin));
                B = Math.round(B_min + (B_max - B_min) * (color - $o.colorSignalMin) / ($o.colorSignalMax - $o.colorSignalMin));
                X = rgbToHex(R, G, B);
                that.element.style.fill = X;
            } else if (color <= $o.colorSignalMin) {
                that.element.style.fill = $o.colorMin;
            } else if (color >= $o.colorSignalMax) {
                that.element.style.fill = $o.colorMax;
            } else {
                that.element.style.fill = "grey";
            }
            return;
        } else if (typeof color === 'string') {
            that.element.style.fill = color;
        }
    };

    that.rotate = function (angle) {
        var oldVal, newVal;

        var angleR = rotationR * (angle - rotationSMin) / (rotationSMax - rotationSMin),
            str = that.element.getAttribute('transform');
        oldVal = getOldValue('rotate', str);
        newVal = 'rotate(' + angleR + ',' + (elemCenterX + rotOffsetX) + ',' + (elemCenterY - rotOffsetY) + ')';
        str = str.replace(oldVal, newVal);
        that.element.setAttribute('transform', str);
    };

    that.translate = function (val, axis) {
        var startI, endI, toReplace, newVal, oldVal;
        if (axis === 'x') {
            var tXa = txR * (val - txSMin) / (txSMax - txSMin),
                str = that.element.getAttribute('transform'),
                oldVal = getOldValue('translate', str);
            if (oldVal === '') {
                newVal = 'translate(' + tXa + ',0)';
            } else {
                startI = oldVal.search('\\(');
                endI = oldVal.search(',');
                toReplace = oldVal.slice(startI + 1, endI + 1);
                newVal = oldVal.replace(toReplace, tXa + ',');
            }
        } else {
            var tYa = tyR * (val - tySMin) / (tySMax - tySMin),
                str = that.element.getAttribute('transform'),
                oldVal = getOldValue('translate', str);
            if (oldVal === '') {
                newVal = 'translate(0,' + -tYa + ')';
            } else {
                startI = oldVal.search(',');
                endI = oldVal.search('\\\)');
                toReplace = oldVal.slice(startI, endI);
                newVal = oldVal.replace(toReplace, ',' + tYa);
            }
        }

        str = str.replace(oldVal, newVal);
        that.element.setAttribute('transform', str);
    };

    that.scale = function (scale) {
        if (scaleX || scaleY) {
            var oldVal, newVal;
            var scaleA = parseFloat(scaleMin) + (scaleMax - scaleMin) * (scale - scaleSMin) / (scaleSMax - scaleSMin),
                str = that.element.getAttribute('transform');
            oldVal = getOldValue('matrix', str);
            if (scaleX && !scaleY) {
                newVal = 'matrix(' + scaleA + ',0,0,1,' + (1 - scaleA) * (elemCenterX + rotOffsetX) + ',0)';
            }
            if (!scaleX && scaleY) {
                newVal = 'matrix(1,0,0,' + scaleA + ',0,' + (1 - scaleA) * (elemCenterY - rotOffsetY) + ')';
            }
            if (scaleX && scaleY) {
                newVal = 'matrix(' + scaleA + ',0,0,' + scaleA + ',' + (1 - scaleA) * (elemCenterX + rotOffsetX) + ',' + (1 - scaleA) * (elemCenterY - rotOffsetY) + ')';
            }
            str = str.replace(oldVal, newVal);
            that.element.setAttribute('transform', str);
        }
    };

    that.opacity = function (opacity) {
        var opacityA = parseFloat(opacityMin) + opacityR * (opacity - opacitySMin) / (opacitySMax - opacitySMin);
        that.element.setAttribute('opacity', Math.abs(opacityA));
        if (opacityA <= 0) {
            $(that.element).css('pointer-events', 'none');
        } else $(that.element).css('pointer-events', 'auto');
    };

    if (typeof that.$c.COLOR !== "undefined") {
        removeChildrenColor();
        that.element.style.fill = "grey";
        that.$c.COLOR.on('change', function (i) {
            that.changeColor(i.getValue());
            that.emit('color');
        });
    }
    if (typeof that.$c.ROTATE !== "undefined") {
        that.$c.ROTATE.on('change', function (i) {
            that.rotate(i.getValue());
            that.emit('rotate');
        });
    }
    if (typeof that.$c.TRANSLATE_X !== "undefined") {
        that.$c.TRANSLATE_X.on('change', function (i) {
            that.translate(i.getValue(), 'x');
            that.emit('translate-x');
        });
    }
    if (typeof that.$c.TRANSLATE_Y !== "undefined") {
        that.$c.TRANSLATE_Y.on('change', function (i) {
            that.translate(i.getValue(), 'y');
            that.emit('translate-y');
        });
    }
    if (typeof that.$c.SCALE !== "undefined") {
        that.$c.SCALE.on('change', function (i) {
            that.scale(i.getValue());
            that.emit('scale');
        });
    }
    if (typeof that.$c.OPACITY !== "undefined") {
        that.$c.OPACITY.on('change', function (i) {
            that.opacity(i.getValue());
            that.emit('opacity');
        });
    }
    function getOldValue(option, string) {
        var startVal = string.search(option + '\\(');
        if (startVal > -1) {
            string = string.slice(startVal);
            var endVal = string.search('\\\)');
            return string.slice(0, endVal + 1);
        } else return '';
    }

    function removeChildrenColor() {
        var childNodes = Array.prototype.slice.call(that.element.childNodes);

        for (var i = 0; i < childNodes.length; i++) {
            var item = childNodes[i];
            if (item.getAttribute) {
                item.style.fill = '';
            }
            childNodes = childNodes.concat(Array.prototype.slice.call(item.childNodes));
        }
    }
    function rgbToHex(R, G, B) {
        return '#' + toHex(R) + toHex(G) + toHex(B);
    }
    function toHex(n) {
        n = parseInt(n, 10);
        if (isNaN(n)) return "00";
        n = Math.max(0, Math.min(n, 255));
        return "0123456789ABCDEF".charAt((n - n % 16) / 16) + "0123456789ABCDEF".charAt(n % 16);
    }
    ;
    function hexToR(h) {
        return parseInt(cutHex(h).substring(0, 2), 16);
    }
    ;
    function hexToG(h) {
        return parseInt(cutHex(h).substring(2, 4), 16);
    }
    ;
    function hexToB(h) {
        return parseInt(cutHex(h).substring(4, 6), 16);
    }
    ;
    function cutHex(h) {
        return h.charAt(0) === "#" ? h.substring(1, 7) : h;
    }
    ;

    // Refresh component to be shown in Firefox
    that.hide();
    setTimeout(function () {
        that.show();
    }, 100);
    return that;
};
'use strict';

REX.UI.SVG.ImageChanger = function (svgElem, args) {
	// Inherit from base component
	var that = new REX.UI.SVG.Component(svgElem, args);
	var $o = that.options || {};
	var imagePath = that.check($o.imagePath, './image.png');
	var appendKey = that.parseBoolean($o.appendKey);

	// Trend
	var svgImage = SVG.adopt(that.getChildByTag("image")); //image

	that.$c.refresh_from.on('change', function (itm) {
		var path = imagePath;
		if (appendKey) {
			path = path.replace('{0}', itm.getValue());
		}
		path += '?version=' + new Date().getTime().toString(32); // Add some hash to refresh image
		svgImage.attr('xlink:href', path);
	});

	return that;
};
'use strict';

/**
 * SVG component represents numeric input.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT} 
 * @returns {REX.UI.SVG.Fan} New SVG numeric input component
 */
REX.UI.SVG.Input = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.HTMLComponent(svgElem, args);
    var $o = that.options || {};

    var format = ($o.format || '').toLowerCase();
    var fontScale = parseFloat(that.check($o.fontScale, 1));
    var min = validateNumericOption('min', $o, -Number.MAX_VALUE);
    var max = validateNumericOption('max', $o, Number.MAX_VALUE);
    var scale = validateNumericOption('scale', $o, 1);
    var offset = validateNumericOption('offset', $o, 0);
    var decimals = validateNumericOption('decimals', $o, 2);
    var setOnBlur = false; // TODO: Support set onBlur functionality

    var virtualKeyboard = that.parseBoolean($o.virtualKeyboard, false);

    // value_R and value_W remains for backward compatibility
    that.$c.refresh_from = that.$c.refresh_from || that.$c.value_R || that.$c.value || that.$c.value_W;
    that.$c.value = that.$c.value || that.$c.value_W;

    that.div.setAttribute('class', 'ui-spinner ui-state-default ui-widget ui-widget-content ui-corner-all');

    var input = $(document.createElement('input'));
    input.attr('type', 'text');
    input.attr('value', 'NaN');
    input.attr('name', 'value');
    input.attr('class', 'ui-spinner-input');
    input.css('width', '100%');
    input.css('height', '100%');
    input.css('margin', '0 0 0 .2em');

    $(that.div).append(input);

    // Init font autoresize
    function updateFontSize() {
        var ctm = that.svg.getScreenCTM();
        // Scale according the width or height which is better
        input.css('font-size', Math.min(ctm.a, ctm.d) * fontScale + 'em');
    }

    updateFontSize();
    $(window).resize(function () {
        updateFontSize();
    });

    function validateNumericOption(name, options, defaultValue) {
        if (options[name] && REX.HELPERS.isNumber(options[name])) {
            return Number(options[name]);
        }
        if (options[name] && !REX.HELPERS.isNumber(options[name])) {
            that.log.error(options.alias + ": " + name + " is not a number. Set to " + defaultValue);
            options[name] = defaultValue;
            return options[name];
        }
        if (options[name] !== "" && options[name] == 0) {
            return options[name];
        } else {
            return defaultValue;
        }
    }

    if (that.$c.value.type === 'R') {
        that.log.error('Connection String: ' + that.$c.value.cstring + '(' + that.$c.value.alias + ') is read-only');
        return that;
    }

    var refresh_from = that.$c.refresh_from;
    var dateSelected = false;

    switch (format) {
        case 'date':
            input.datepicker({
                dateFormat: 'dd.mm.yy',
                onSelect: function onSelect() {
                    if (!that.isDisabled()) {
                        that.$c.value.write(that.getREXSecondsFromDate(input.datepicker("getDate"))).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        updateRefreshFromItemAfterWrite();
                    }
                }
            });
            break;
        case 'time':
            input.timepicker({
                // defautValue:"NaN",
                parse: "loose",
                alwaysSetTime: false,
                timeFormat: "HH:mm",
                onOpen: function onOpen() {
                    refresh_from.disableRefresh = true;
                },
                onSelect: function onSelect(date) {
                    dateSelected = true;
                },
                onClose: function onClose(date) {
                    // Save only if date was selected
                    if (date && dateSelected && !that.isDisabled()) {
                        that.$c.value.write(that.getREXSecondsFromDate(input.datetimepicker("getDate"))).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        updateRefreshFromItemAfterWrite();
                    }
                    dateSelected = false;
                    refresh_from.disableRefresh = false;
                }
            });
            break;
        case 'datetime':
            input.datetimepicker({
                //defautValue:"NaN",
                parse: "loose",
                alwaysSetTime: false,
                dateFormat: 'dd.mm.yy',
                timeFormat: 'HH:mm',
                onOpen: function onOpen() {
                    that.disableRefresh = true;
                },
                onSelect: function onSelect(date) {
                    dateSelected = true;
                },
                onClose: function onClose(date) {
                    // Save only if date was selected
                    if (date && dateSelected && !that.isDisabled()) {
                        that.$c.value.write(that.getREXSecondsFromDate(input.datetimepicker("getDate"))).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        updateRefreshFromItemAfterWrite();
                    }
                    dateSelected = false;
                    that.disableRefresh = false;
                }
            });

            break;
        default:
            input.focus(function (evt) {
                that.disableRefresh = true;
            }).blur(function (evt) {
                that.disableRefresh = false;
                if (refresh_from) {
                    refresh_from.emit('change', refresh_from);
                }
            });
            input.keypress(function (evt) {
                var keyCode = evt.keyCode || evt.which;
                if (keyCode === 13) {
                    //Enter keycode
                    if (format === 'text') {
                        that.$c.value.write(input.prop('value')).catch(function (err) {
                            that.log.error("Write failed", err);
                        });
                        updateRefreshFromItemAfterWrite();
                    } else {
                        var numberStr = input.prop('value').replace(',', '.');
                        if (REX.HELPERS.isNumber(numberStr)) {
                            var number = Number(numberStr);
                            if (number >= min && number <= max) {
                                input.removeClass('ui-state-error');
                                if (!that.isDisabled()) {
                                    number = (number - offset) / scale;
                                    that.$c.value.write(number).catch(function (err) {
                                        that.log.error("Write failed", err);
                                    });
                                    updateRefreshFromItemAfterWrite();
                                }
                            } else {
                                // TODO: Show tootltip with range
                                input.addClass('ui-state-error');
                                return;
                            }
                        } else {
                            input.addClass('ui-state-error');
                        }
                    }
                }
            });
            input.keyup(function (evt) {
                var keyCode = evt.keyCode || evt.which;
                if (keyCode === 27) {
                    //ESC keycode
                    var setOnBlurTmp = setOnBlur;
                    // Temporary disable set on blur behaviour for ESC key
                    setOnBlur = false;
                    input.blur();
                    setOnBlur = setOnBlurTmp;
                }
            });
            break;
    }

    function updateRefreshFromItemAfterWrite() {
        // Temporary save the same value to the refreshfrom item
        if (refresh_from) {
            refresh_from.setValue(that.$c.value.getValue());
        }
    }

    function refresh(itm) {
        switch (format) {
            case 'date':
                input.datepicker("setDate", that.getDateFromREXSeconds(itm.value));
                break;
            case 'time':
                input.datetimepicker('setDate', that.getDateFromREXSeconds(itm.value));
                break;
            case 'datetime':
                input.datetimepicker("setDate", that.getDateFromREXSeconds(itm.value));
                break;
            case 'text':
                input.prop("value", '' + itm.value);
                break;
            default:
                var resultValue = itm.value * scale + offset;
                input.prop("value", '' + resultValue.toFixed(decimals));
                break;
        }
    }

    refresh_from.on('change', function (itm) {
        if (!refresh_from.disableRefresh) {
            refresh(itm);
        }
    });

    // Override disable and enable function
    var old_disable = that.disable;
    that.disable = function () {
        input.addClass('ui-state-disabled');
        old_disable.apply(this, arguments);
    };
    var old_enable = that.enable;
    that.enable = function () {
        input.removeClass('ui-state-disabled');
        old_enable.apply(this, arguments);
    };

    if (virtualKeyboard) {
        input.keyboard({
            // *** choose layout ***
            layout: 'custom',
            // Qwerty only with Accept key
            customLayout: {
                default: ['7 8 9 {b}', '4 5 6 -', "1 2 3 +", '0 . {clear} {a}']
            },
            //        position: {
            //            of: '#Input-div input', // optional - null (attach to input/textarea) or a jQuery object (attach elsewhere)
            //            my: 'center top',
            //            at: 'center top'
            //        },
            // preview added above keyboard if true, original input/textarea used if false
            usePreview: false,
            // if true, the keyboard will always be visible
            alwaysOpen: false,
            // give the preview initial focus when the keyboard becomes visible
            initialFocus: false,
            // if true, keyboard will remain open even if the input loses focus.
            stayOpen: false,
            // *** Useability ***
            // Auto-accept content when clicking outside the keyboard (popup will close)
            autoAccept: false,
            // Prevents direct input in the preview window when true
            lockInput: false,
            // Prevent keys not in the displayed keyboard from being typed in
            restrictInput: false,
            // Check input against validate function, if valid the accept button is clickable;
            // if invalid, the accept button is disabled.
            acceptValid: false,
            // Use tab to navigate between input fields
            tabNavigation: false,
            // press enter (shift-enter in textarea) to go to the next input field
            enterNavigation: false,
            // mod key options: 'ctrlKey', 'shiftKey', 'altKey', 'metaKey' (MAC only)
            enterMod: 'altKey', // alt-enter to go to previous; shift-alt-enter to accept & go to previous

            // if true, the next button will stop on the last keyboard input/textarea; prev button stops at first
            // if false, the next button will wrap to target the first input/textarea; prev will go to the last
            stopAtEnd: true,
            // Set this to append the keyboard immediately after the input/textarea it is attached to.
            // This option works best when the input container doesn't have a set width and when the
            // "tabNavigation" option is true
            appendLocally: false,
            // If false, the shift key will remain active until the next key is (mouse) clicked on;
            // if true it will stay active until pressed again
            stickyShift: true,
            // Prevent pasting content into the area
            preventPaste: false,
            // Set the max number of characters allowed in the input, setting it to false disables this option
            maxLength: false,
            // Mouse repeat delay - when clicking/touching a virtual keyboard key, after this delay the key
            // will start repeating
            repeatDelay: 500,
            // Mouse repeat rate - after the repeatDelay, this is the rate (characters per second) at which the
            // key is repeated. Added to simulate holding down a real keyboard key and having it repeat. I haven't
            // calculated the upper limit of this rate, but it is limited to how fast the javascript can process
            // the keys. And for me, in Firefox, it's around 20.
            repeatRate: 20,
            // resets the keyboard to the default keyset when visible
            resetDefault: false,
            // Event (namespaced) on the input to reveal the keyboard. To disable it, just set it to ''.
            openOn: 'focus',
            // When the character is added to the input
            keyBinding: 'mousedown touchstart',
            // combos (emulate dead keys : http://en.wikipedia.org/wiki/Keyboard_layout#US-International)
            // if user inputs `a the script converts it to à, ^o becomes ô, etc.
            useCombos: false,
            accepted: function accepted(e, keyboard, el) {
                input.trigger($.Event('keypress', { keyCode: 13 }));
            },
            visible: function visible(e, keyboard, el) {
                // Clear value on open
                keyboard.$preview.val("");
            }
        });
    }

    that.disable();
    return that;
};
"use strict";

/**
 * SVG component represents Led.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Led} New SVG Led component
 */

REX.UI.SVG.Led = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var colorRun = $o.colorRun || "#33ee00";
    var colorStop = $o.colorStop || "ff2200";

    // Get SVG elements for manipulation
    var led = that.getChildByTag("led");

    // Add anonymous function as event listener. There are two events
    // 'read' - it is called every time when item is read
    // 'change' - called for the first time and every time item value is changed
    that.$c.POWER.on('change', function (itm) {
        if (itm.getValue()) {
            led.setAttributeNS(null, "style", "fill:" + colorRun + "; fill-opacity:1.0;fill-rule:nonzero;stroke:none)");
        } else {
            led.setAttributeNS(null, "style", "fill:" + colorStop + "; fill-opacity:1.0;fill-rule:nonzero;stroke:none)");
        }
    });

    return that;
};
"use strict";

/**
 * Status component with label
 * @param svgElem
 * @param args
 * @returns {*}
 * @constructor
 */
REX.UI.SVG.LedLabel = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Load options or default values
    var label_false = $o.label_false || ($o.reverse_meaning ? "NOK(1)" : "NOK(0)");
    var label_true = $o.label_true || ($o.reverse_meaning ? "OK(0)" : "OK(1)");
    var reverse_meaning = $o.reverse_meaning || false;
    var color_false = $o.color_false || "#E74E3E";
    var color_true = $o.color_true || "#46B281";

    // Get SVG elements for manipulation
    var svgLabel = that.getChildByTag("label"); //tspan
    var svgLed = that.getChildByTag("led"); //rect


    that.$c.STATUS.on('change', function (itm) {
        if (itm.value == reverse_meaning) {
            svgLabel.textContent = label_false;
            if (color_false) {
                $(svgLed).css('fill', color_false);
            }
        } else {
            svgLabel.textContent = label_true;
            if (color_true) {
                $(svgLed).css('fill', color_true);
            }
        }
    });

    return that;
};
'use strict';

/**
 * SVG component represents PushOnOff.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.PushOnOff} New SVG PushOnOff component
 */

REX.UI.SVG.PushOnOff = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Get options or default values
    var reverse_meaning = that.parseBoolean($o.reverse_meaning);
    var type = $o.type || 'PushButton';

    // value_R and value_W remains for backward compatibility
    that.$c.refresh_from = that.$c.refresh_from || that.$c.VALUE_R || that.$c.BUTTON || that.$c.VALUE_W;

    if (!that.$c.BUTTON.writeCString) {
        that.log.error('Item %s is read-only!', that.$c.BUTTON.id);
    }

    // Get SVG elements for manipulation
    var pushOnOffArea = that.getChildByTag("pushOnOff_area"),
        buttonP1 = that.getChildByTag("button_part1"),
        buttonP2 = that.getChildByTag("button_part2"),
        buttonP3 = that.getChildByTag("button_part3");

    //Global variables
    var centerX = pushOnOffArea.getBBox().width / 2,
        centerY = pushOnOffArea.getBBox().height / 2,
        currentPosition = 0;

    //pushOnOffArea.addEventListener("click", switchValue, false);
    pushOnOffArea.addEventListener("mouseover", mOver, false);
    pushOnOffArea.addEventListener("mouseout", mOut, false);
    pushOnOffArea.addEventListener("mousedown", mDown, false);
    pushOnOffArea.addEventListener("mouseup", mOut, false);

    function setValue(val) {
        if (val === 0) {
            that.$c.BUTTON.write(reverse_meaning).catch(function (err) {
                that.log.error("Write failed", err);
            });
            currentPosition = 0;
        } else {
            that.$c.BUTTON.write(!reverse_meaning).catch(function (err) {
                that.log.error("Write failed", err);
            });
            currentPosition = 1;
        }
    }

    function mOver(event) {
        if (currentPosition == 0) {
            buttonP1.style.fill = "#838383";
            buttonP3.style.fill = "#838383";
        } else {
            buttonP1.style.fill = "#33bb22";
            buttonP3.style.fill = "#33bb22";
        }
    }

    function mOut(event) {
        that.$c.refresh_from.disableRefresh = false;
        if (type === 'PushButton') {
            setValue(0);
        }
        if (currentPosition == 0) {
            buttonP1.style.fill = "#b3b3b3";
            buttonP3.style.fill = "#b3b3b3";
        } else {
            buttonP1.style.fill = "#33ff00";
            buttonP3.style.fill = "#33ff00";
        }
    }

    function mDown(event) {
        that.$c.refresh_from.disableRefresh = true;
        if (type === 'ToggleButton') {
            var val = currentPosition ? 0 : 1; // Inverse value
            setValue(val);
        } else {
            setValue(1);
        }
        if (currentPosition === 0) {
            buttonP1.style.fill = "#636363";
            buttonP3.style.fill = "#636363";
        } else {
            buttonP1.style.fill = "#338811";
            buttonP3.style.fill = "#338811";
        }
    }

    that.$c.refresh_from.on('change', function onChange(itm) {
        if (!that.$c.refresh_from.disableRefresh) {
            if (reverse_meaning) {
                currentPosition = itm.getValue() === 0 ? 1 : 0;
            } else {
                currentPosition = itm.getValue();
            }
            if (currentPosition === 0) {
                buttonP1.style.fill = "#b3b3b3";
                buttonP3.style.fill = "#808080";
            } else {
                buttonP1.style.fill = "#33ff00";
                buttonP3.style.fill = "#33aa22";
            }
        }
    });

    return that;
};
"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * SVG component represents SimpleLogger.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT} 
 * @returns {SimpleLogger} New HTML component
 */
REX.UI.SVG.SimpleLogger = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.HTMLComponent(svgElem, args);
    var $div = $(that.div);

    var $o = that.options || {};
    var lines = that.checkNumber($o.lines, 1);
    var timestamp = that.parseBoolean($o.timestamp);
    var showKey = that.parseBoolean($o.showKey);
    var format = $o.format || "alt";

    $div.css({ "white-space": "pre-wrap", "overflow-x": "hidden", "overflow-y": "auto", "text-align": "left" });

    if ($o.css) {
        if (_typeof($o.css) === 'object') {
            $div.css($o.css);
        } else {
            that.log.error(that.id + "css property is not an object. Write it as a value pair JSON object");
        }
    }

    var texts = {};
    // If text is defined than convert array to object where key is id (number)
    // and value is text
    if ($o.texts && $o.texts.length > 0) {
        var result = {};
        for (var i = 0; i < $o.texts.length; i++) {
            result[$o.texts[i].id] = $o.texts[i].text;
        }
        texts = result;
    }

    that.$c.value.on('change', function (itm) {
        var log_msg = '<div>';
        if (timestamp) {
            var actDate = new Date();
            var hours = actDate.getHours() < 10 ? "0" + actDate.getHours() : "" + actDate.getHours();
            var minutes = actDate.getMinutes() < 10 ? "0" + actDate.getMinutes() : "" + actDate.getMinutes();
            var seconds = actDate.getSeconds() < 10 ? "0" + actDate.getSeconds() : "" + actDate.getSeconds();
            log_msg += '<span class="logger-timestamp"><b>' + hours + ':' + minutes + ':' + seconds + '</b></span>';
        }
        if (showKey) {
            log_msg += "<span class='logger-key'>" + itm.getValue() + "</span>";
        }
        log_msg += "<span class='logger-value'>";
        switch (format.toLowerCase()) {
            case 'text':
                log_msg += itm.getValue();
                break;
            default:
                if (texts[itm.getValue()]) log_msg += texts[itm.getValue()];else {
                    log_msg += itm.getValue();
                }
                break;
        }
        log_msg += '</span></div>';
        $div.prepend(log_msg);
        if ($div.children().length > lines) {
            $div.children().last().remove();
        }
        //$div.scrollTop($div[0].scrollHeight);
    });

    // Override disable and enable function
    var old_disable = that.disable;
    that.disable = function () {
        $div.addClass('ui-state-disabled');
        old_disable.apply(this, arguments);
    };
    var old_enable = that.enable;
    that.enable = function () {
        $div.removeClass('ui-state-disabled');
        old_enable.apply(this, arguments);
    };

    that.disable();
    return that;
};
"use strict";

/**
 * SVG component represents Slider.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Slider} New SVG Slider component
 */

REX.UI.SVG.SliderHorizontal = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // value_R and value_W remains for backward compatibility
    that.$c.refresh_from = that.$c.refresh_from || that.$c.value_R || that.$c.value || that.$c.value_W;
    that.$c.value = that.$c.value || that.$c.value_W;

    // Get SVG elements for manipulation
    var sliderArea = that.getChildByTag("slider_area"),
        level = that.getChildByTag("slider_level"),
        levelBox = that.getChildByTag("slider_capacity"),
        dragPoint = that.getChildByTag("drag_point"),
        digitalValue = that.getChildByTag("digitalval"),
        textBox = that.getChildByTag("text_box");
    // Deprecated warning
    if (!textBox) {
        that.log.warn(that.id + ': Please upgrade this component');
    }

    //Load options or default values
    var min = parseFloat(that.check($o.min, 0)),
        max = parseFloat(that.check($o.max, 100)),
        step = parseFloat(that.check($o.step, 1)),
        scale = parseFloat(that.check($o.scale, 1)),
        offset = parseFloat(that.check($o.offset, 0)),
        decimals = parseFloat(that.check($o.decimals, 0)),
        fScale = parseFloat(that.check($o.fontScale, 1)),
        fSize = fScale * 18,
        label = $o.label || "";
    digitalValue.style.fontSize = fScale * 18 + "px";
    createLabel();

    //Global variables
    var setPoint,
        setPointChanged = false,
        sliderActive = false,
        activeArea = createActiveArea(),
        activePoint = activeArea.createSVGPoint();

    sliderArea.addEventListener("mousedown", sliderDown, false);
    document.addEventListener("mouseup", sliderUp, false);
    document.addEventListener("mousemove", sliderMove, false);
    sliderArea.addEventListener("click", sliderClick, false);

    that.$c.refresh_from.on('change', function (itm) {
        if (!that.$c.refresh_from.disableRefresh) {
            updateSlider(itm.getValue() * scale + offset);
        }
    });

    function sliderDown(event) {
        sliderActive = true;
        that.$c.refresh_from.disableRefresh = true;
        updatePosition(event);
    }

    function sliderMove(event) {
        if (sliderActive) {
            updatePosition(event);
        }
    }

    function sliderClick(event) {
        updatePosition(event);
    }

    function sliderUp(event) {
        sliderActive = false;
        that.$c.refresh_from.disableRefresh = false;
        var value = (parseFloat(setPoint) - offset) / scale;
        if (setPointChanged) {
            that.$c.value.write(value).catch(function (err) {
                that.log.error("Write failed", err);
            });
            if (that.$c.value !== that.$c.refresh_from) {
                that.$c.refresh_from.setValue(value);
            }
            setPointChanged = false;
        }
    }

    function updatePosition(event) {
        activePoint.x = event.pageX;
        activePoint.y = event.pageY;
        var newPoint = coordinateTransform(activePoint, activeArea);
        var position = newPoint.x;
        setValue(position);
        if (sliderActive && position > -1 && position < levelBox.getBBox().width + 1) {
            dragPoint.setAttributeNS(null, "transform", "translate(" + parseFloat(position) + "," + 0 + ")");
            level.setAttributeNS(null, "width", position);
            digitalValue.textContent = setPoint.toFixed(decimals);
            transformDisplay();
            setPointChanged = true;
        }
    }

    function setValue(val) {
        var relativeStep = levelBox.getBBox().width / (Math.abs(max - min) / step);
        if (val % relativeStep < relativeStep) {
            setPoint = min + Math.round(val / relativeStep) * step;
            if (setPoint < min) {
                setPoint = min;
            } else if (setPoint > max) {
                setPoint = max;
            }
        }
    }

    function updateSlider(setPointValue) {
        var setP = setPointValue;
        var position;
        if (setP >= min && setP <= max) {
            position = (setP - min) * levelBox.getBBox().width / Math.abs(max - min);
        } else {
            if (setP < min) {
                setP = min;
                position = 0;
            } else {
                setP = max;
                position = levelBox.getBBox().width;
            }
        }
        digitalValue.textContent = setP.toFixed(decimals);
        transformDisplay();
        dragPoint.setAttributeNS(null, "transform", "translate(" + parseFloat(position) + "," + 0 + ")");
        level.setAttributeNS(null, "width", position);
    }

    function coordinateTransform(screenPoint, someSvgObject) {
        var CTM = someSvgObject.getScreenCTM();
        if (/Firefox[\/\s](\d+\.\d+)/.test(navigator.userAgent)) {
            var newCoordinates = screenPoint.matrixTransform(CTM.inverse());
            newCoordinates.x -= levelBox.getBBox().x;
            return newCoordinates;
        } else {
            return screenPoint.matrixTransform(CTM.inverse());
        }
    }

    function createActiveArea() {
        var area = document.createElementNS("http://www.w3.org/2000/svg", 'svg');
        area.setAttribute('x', levelBox.getBBox().x);
        area.setAttribute('y', levelBox.getBBox().y);
        area.setAttribute('width', levelBox.getBBox().width);
        area.setAttribute('height', levelBox.getBBox().height);
        sliderArea.appendChild(area);
        return area;
    }

    function createLabel() {
        var text = document.createElementNS("http://www.w3.org/2000/svg", "text");
        text.setAttributeNS(null, "fill", "#ffffff");
        text.setAttributeNS(null, "style", "font-size:" + fSize + "px; font-family:Arial");
        text.textContent = label;
        sliderArea.appendChild(text);
        text.setAttributeNS(null, "x", levelBox.getBBox().x + levelBox.getBBox().width / 2 - text.getBBox().width / 2);
        text.setAttributeNS(null, "y", levelBox.getBBox().y + levelBox.getBBox().height + text.getBBox().height * 1.1);
    }

    function transformDisplay() {
        if (textBox) {
            var fontSize = digitalValue.style.fontSize;
            if (digitalValue.parentNode.getBBox().width >= textBox.getBBox().width * 0.9) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.9 + "px";
            } else if (digitalValue.parentNode.getBBox().width < textBox.getBBox().width * 0.85 && parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 < fSize) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 + "px";
            }
            digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(textBox.getBBox().x + (textBox.getBBox().width / 2 - digitalValue.parentNode.getBBox().width / 2) - digitalValue.parentNode.getBBox().x) + "," + parseInt(textBox.getBBox().y + (textBox.getBBox().height / 2 - digitalValue.parentNode.getBBox().height / 2) - digitalValue.parentNode.getBBox().y) + ")");
        }
    }

    return that;
};
"use strict";

/**
 * SVG component represents Slider.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Slider} New SVG Slider component
 */

REX.UI.SVG.SliderVertical = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // value_R and value_W remains for backward compatibility
    that.$c.refresh_from = that.$c.refresh_from || that.$c.value_R || that.$c.value || that.$c.value_W;
    that.$c.value = that.$c.value || that.$c.value_W;

    // Get SVG elements for manipulation
    var sliderArea = that.getChildByTag("slider_area"),
        level = that.getChildByTag("slider_level"),
        levelBox = that.getChildByTag("slider_capacity"),
        dragPoint = that.getChildByTag("drag_point"),
        digitalValue = that.getChildByTag("digitalval"),
        textBox = that.getChildByTag("text_box");

    // Deprecated warning
    if (!textBox) {
        that.log.warn(that.id + ': Please upgrade this component');
    }

    //Load options or default values
    var min = parseFloat(that.check($o.min, 0)),
        max = parseFloat(that.check($o.max, 100)),
        step = parseFloat(that.check($o.step, 1)),
        scale = parseFloat(that.check($o.scale, 1)),
        offset = parseFloat(that.check($o.offset, 0)),
        decimals = parseFloat(that.check($o.decimals, 0)),
        fScale = parseFloat(that.check($o.fontScale, 1)),
        fSize = fScale * 18,
        label = $o.label || "";
    digitalValue.style.fontSize = fScale * 18 + "px";
    createLabel();
    level.setAttributeNS(null, "transform", "rotate(" + 180 + "," + (levelBox.getBBox().x + levelBox.getBBox().width / 2) + "," + (levelBox.getBBox().y + levelBox.getBBox().height) + ")");

    //Global variables
    var setPoint,
        setPointChanged = false,
        sliderActive = false,
        activeArea = createActiveArea(),
        activePoint = activeArea.createSVGPoint();

    sliderArea.addEventListener("mousedown", sliderDown, false);
    document.addEventListener("mouseup", sliderUp, false);
    document.addEventListener("mousemove", sliderMove, false);
    sliderArea.addEventListener("click", sliderClick, false);

    that.$c.refresh_from.on('change', function (itm) {
        if (!that.$c.refresh_from.disableRefresh) {
            updateSlider(itm.getValue() * scale + offset);
        }
    });

    function sliderDown(event) {
        sliderActive = true;
        that.$c.refresh_from.disableRefresh = true;
        updatePosition(event);
    }

    function sliderMove(event) {
        if (sliderActive) {
            updatePosition(event);
        }
    }

    function sliderClick(event) {
        updatePosition(event);
    }

    function sliderUp(event) {
        sliderActive = false;
        that.$c.refresh_from.disableRefresh = false;
        var value = (parseFloat(setPoint) - offset) / scale;
        if (setPointChanged) {
            that.$c.value.write(value).catch(function (err) {
                that.log.error("Write failed", err);
            });
            if (that.$c.value !== that.$c.refresh_from) {
                that.$c.refresh_from.setValue(value);
            }
            setPointChanged = false;
        }
    }

    function updatePosition(event) {
        activePoint.x = event.pageX;
        activePoint.y = event.pageY;
        var newPoint = coordinateTransform(activePoint, activeArea);
        var position = levelBox.getBBox().height - newPoint.y;
        setValue(position);
        if (sliderActive && position > -1 && position < levelBox.getBBox().height + 1) {
            dragPoint.setAttributeNS(null, "transform", "translate(" + 0 + "," + -parseFloat(position) + ")");
            level.setAttributeNS(null, "height", position);
            digitalValue.textContent = setPoint.toFixed(decimals);
            transformDisplay();
            setPointChanged = true;
        }
    }

    function setValue(val) {
        var relativeStep = levelBox.getBBox().height / (Math.abs(max - min) / step);
        if (val % relativeStep < relativeStep) {
            setPoint = min + Math.round(val / relativeStep) * step;
            if (setPoint < min) {
                setPoint = min;
            } else if (setPoint > max) {
                setPoint = max;
            }
        }
    }

    function updateSlider(setPointValue) {
        var setP = setPointValue;
        var position;
        if (setP >= min && setP <= max) {
            position = (setP - min) * levelBox.getBBox().height / Math.abs(max - min);
        } else {
            if (setP < min) {
                setP = min;
                position = 0;
            } else {
                setP = max;
                position = levelBox.getBBox().height;
            }
        }
        digitalValue.textContent = setP.toFixed(decimals);
        transformDisplay();
        dragPoint.setAttributeNS(null, "transform", "translate(" + 0 + "," + -parseFloat(position) + ")");
        level.setAttributeNS(null, "height", position);
    }

    function coordinateTransform(screenPoint, someSvgObject) {
        var CTM = someSvgObject.getScreenCTM();
        if (/Firefox[\/\s](\d+\.\d+)/.test(navigator.userAgent)) {
            var newCoordinates = screenPoint.matrixTransform(CTM.inverse());
            newCoordinates.y -= levelBox.getBBox().y;
            return newCoordinates;
        } else {
            return screenPoint.matrixTransform(CTM.inverse());
        }
    }

    function createActiveArea() {
        var area = document.createElementNS("http://www.w3.org/2000/svg", 'svg');
        area.setAttribute('x', levelBox.getBBox().x);
        area.setAttribute('y', levelBox.getBBox().y);
        area.setAttribute('width', levelBox.getBBox().width);
        area.setAttribute('height', levelBox.getBBox().height);
        sliderArea.appendChild(area);
        return area;
    }

    function createLabel() {
        var text = document.createElementNS("http://www.w3.org/2000/svg", "text");
        var oldWidth = sliderArea.getBBox().width;
        text.setAttributeNS(null, "fill", "#ffffff");
        text.setAttributeNS(null, "style", "font-size:" + fSize + "px; font-family:Arial");
        text.textContent = label;
        sliderArea.appendChild(text);
        // Resize font size if needed
        if (text.getBBox().width > oldWidth) {
            var newFontSize = oldWidth / text.getBBox().width * fSize * 0.98;
            text.setAttributeNS(null, "style", "font-size:" + newFontSize.toFixed(3) + "px; font-family:Arial");
        }
        text.setAttributeNS(null, "x", levelBox.getBBox().x + levelBox.getBBox().width / 2 - text.getBBox().width / 2);
        text.setAttributeNS(null, "y", levelBox.getBBox().y + levelBox.getBBox().height + text.getBBox().height * 1.1);
    }

    function transformDisplay() {
        if (textBox) {
            // For backward compatibility
            var fontSize = digitalValue.style.fontSize;
            if (digitalValue.parentNode.getBBox().width >= textBox.getBBox().width * 0.9) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 0.9 + "px";
            } else if (digitalValue.parentNode.getBBox().width < textBox.getBBox().width * 0.85 && parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 < fSize) {
                digitalValue.style.fontSize = parseFloat(fontSize.substring(0, fontSize.indexOf('p'))) * 1.05 + "px";
            }
            digitalValue.parentNode.setAttributeNS(null, "transform", "translate(" + parseInt(textBox.getBBox().x + (textBox.getBBox().width / 2 - digitalValue.parentNode.getBBox().width / 2) - digitalValue.parentNode.getBBox().x) + "," + parseInt(textBox.getBBox().y + (textBox.getBBox().height / 2 - digitalValue.parentNode.getBBox().height / 2) - digitalValue.parentNode.getBBox().y) + ")");
        }
    }

    return that;
};
"use strict";

/**
 * SVG component represents Switch.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Switch} New SVG Switch component
 */

REX.UI.SVG.Switch = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Get options or default values
    var positions = $o.positions || null;

    that.$c.refresh_from = that.$c.refresh_from || that.$c.value_R || that.$c.value || that.$c.value_W;
    var refresh_from = that.$c.refresh_from;

    that.$c.value = that.$c.value || that.$c.value_W;

    // Get SVG elements for manipulation
    var switchArea = that.getChildByTag("switch_area"),
        hand = that.getChildByTag("hand");

    //Global variables
    var centerX = switchArea.getBBox().width / 2,
        centerY = switchArea.getBBox().height / 2 - 1,
        currentPosition = 0,
        points = new Array();

    var valuesOfPositions = new Array();
    for (var j = 0; j < positions.length; j++) {
        valuesOfPositions[j] = positions[j].valueOfPosition;
    }

    var i = 0;
    var pointAngle = 360 / valuesOfPositions.length;
    while (i < valuesOfPositions.length) {
        createPoint(i);
        i++;
    }

    switchArea.addEventListener("click", switchValueRight, false);
    switchArea.addEventListener("contextmenu", switchValueLeft, false);

    function switchValueRight(event) {
        event.preventDefault();
        currentPosition = (currentPosition + 1) % valuesOfPositions.length;
        that.$c.value.write(parseFloat(valuesOfPositions[currentPosition])).catch(function (err) {
            that.log.error("Write failed", err);
        });
        refresh();
    }

    function switchValueLeft(event) {
        event.preventDefault();
        currentPosition = currentPosition - 1 % valuesOfPositions.length;
        if (currentPosition < 0) currentPosition = valuesOfPositions.length - 1;
        that.$c.value.write(parseFloat(valuesOfPositions[currentPosition])).catch(function (err) {
            that.log.error("Write failed", err);
        });
        refresh();
    }

    function createPoint(i) {
        var x = centerX + Math.sqrt(centerX / 1.6888 * centerX / 1.6888 + centerY / 1.6888 * centerY / 1.6888) * Math.cos((180 - pointAngle * i) * Math.PI / 180);
        var y = centerY - Math.sqrt(centerX / 1.6888 * centerX / 1.6888 + centerY / 1.6888 * centerY / 1.6888) * Math.sin((180 - pointAngle * i) * Math.PI / 180);

        var circle = document.createElementNS("http://www.w3.org/2000/svg", "circle");
        circle.setAttributeNS(null, "cx", x);
        circle.setAttributeNS(null, "cy", y);
        circle.setAttributeNS(null, "r", 1);
        circle.setAttributeNS(null, "fill", "#ffffff");
        circle.setAttributeNS(null, "style", "stroke:none");
        switchArea.appendChild(circle);
        points[i] = circle;
    }

    refresh_from.on('change', function onChange(itm) {
        if (valuesOfPositions.lastIndexOf(itm.getValue().toString()) >= 0) {
            var currentAngle = pointAngle * valuesOfPositions.lastIndexOf(itm.getValue().toString());
            hand.setAttributeNS(null, "transform", "rotate(" + currentAngle + "," + centerX + "," + centerY + ")");
            currentPosition = valuesOfPositions.lastIndexOf(itm.getValue().toString());
            for (var i = 0; i < points.length; i++) {
                points[i].style.fill = "#ffffff";
                points[i].setAttributeNS(null, "r", 1);
            }
            points[currentPosition].style.fill = "#00ff00";
            points[currentPosition].setAttributeNS(null, "r", 2);
        } else {
            for (var i = 0; i < valuesOfPositions.length - 1; i++) {
                if (itm.getValue() > parseFloat(valuesOfPositions[i]) && itm.getValue() < parseFloat(valuesOfPositions[i + 1])) {
                    var currentAngle = pointAngle * i;
                    hand.setAttributeNS(null, "transform", "rotate(" + currentAngle + "," + centerX + "," + centerY + ")");
                    currentPosition = i;
                    for (var j = 0; j < points.length; j++) {
                        points[j].style.fill = "#ffffff";
                        points[j].setAttributeNS(null, "r", 1);
                    }
                    points[currentPosition].style.fill = "#00ff00";
                    points[currentPosition].setAttributeNS(null, "r", 2);
                }
            }
        }
    });

    function refresh() {
        refresh_from.setValue(that.$c.value.getValue());
        refresh_from.emit('change', refresh_from);
    }

    return that;
};
"use strict";

/**
 * SVG component represents Switch.
 * @param {SVGElement} svgElem 
 * @param {Object} args It is possible to specify {type:"",svg:SVG_ELEMENT,defs:DEFS_ELEMENT}
 * @returns {REX.UI.SVG.Switch} New SVG Switch component
 */

REX.UI.SVG.SwitchOnOff = function (svgElem, args) {
    // Inherit from base component
    var that = new REX.UI.SVG.Component(svgElem, args);
    // Store options for simple usage
    var $o = that.options || {};

    // Get options or default values
    var reverse_meaning = that.parseBoolean($o.reverse_meaning);

    that.$c.refresh_from = that.$c.refresh_from || that.$c.VALUE_R || that.$c.value || that.$c.VALUE_W;
    var refresh_from = that.$c.refresh_from;

    that.$c.value = that.$c.value || that.$c.VALUE_W;

    if (!that.$c.value.writeCString) {
        that.log.error('Item %s is read-only!', that.$c.value.id);
    }

    // Get SVG elements for manipulation
    var switchArea = that.getChildByTag("switch_area"),
        hand = that.getChildByTag("hand"),
        numberOne = that.getChildByTag("number_one");

    //Global variables
    var centerX = switchArea.getBBox().width / 2,
        centerY = switchArea.getBBox().height / 2,
        currentPosition,
        init = false;

    switchArea.addEventListener("click", switchValue, false);
    switchArea.addEventListener("contextmenu", switchValue, false);

    function refresh() {
        if (currentPosition === 0) {
            hand.setAttributeNS(null, "transform", "rotate(" + 0 + "," + centerX + "," + centerY + ")");
            numberOne.style.fill = "#ffffff";
        } else {
            hand.setAttributeNS(null, "transform", "rotate(" + 90 + "," + centerX + "," + centerY + ")");
            numberOne.style.fill = "#00ff00";
        }
    }

    function switchValue(event) {
        event.preventDefault();
        if (currentPosition === 0) {
            that.$c.value.write(!reverse_meaning).catch(function (err) {
                that.log.error("Write failed", err);
            });
            currentPosition = 1;
        } else {
            that.$c.value.write(reverse_meaning).catch(function (err) {
                that.log.error("Write failed", err);
            });
            currentPosition = 0;
        }
        refresh();
    }

    refresh_from.on('change', function onChange(itm) {
        if (reverse_meaning) {
            currentPosition = itm.getValue() === 0 ? 1 : 0;
        } else {
            currentPosition = itm.getValue();
        }
        refresh();
    });

    return that;
};